#pragma semicolon 1
#pragma newdecls required

#include <sourcemod>
#include <sdktools_functions>
#include <vip_core>

#define USE_HEALTH
#define USE_ARMOR
#define USE_SPEED
#define USE_MONEY

public Plugin myinfo =
{
	name = "[VIP] Base Functions",
	author = "R1KO, vadrozh",
	description = "Добавляет VIP игрокам доп. единицы жизни, брони, скорости, денег.",
	version = "1.6",
	url = "https://hlmod.ru"
};

#define VIP_HEALTH	0
#define VIP_ARMOR	1
#define VIP_SPEED	2
#define VIP_MONEY	3

static const char g_szFeature[][] = {"HP", "Armor", "Speed", "Money"};

#if defined USE_HEALTH
int m_iHealth;
#endif
#if defined USE_ARMOR
int m_ArmorValue;
int m_bHasHelmet;
#endif
#if defined USE_SPEED
int m_flLaggedMovementValue;
#endif
#if defined USE_MONEY
int m_iAccount;
#endif

int g_iCvar_WaitRounds,
	g_iRounds;

bool g_bIsCSGO, g_bGiveHelmet;

ConVar g_hWaitRounds, g_hGiveHelmet;

public void OnPluginStart()
{
#if defined USE_HEALTH
	m_iHealth				= UTIL_FindSendPropOffs("CCSPlayer", "m_iHealth");
#endif
#if defined USE_ARMOR
	m_ArmorValue			= UTIL_FindSendPropOffs("CCSPlayer", "m_ArmorValue");
	m_bHasHelmet			= UTIL_FindSendPropOffs("CCSPlayer", "m_bHasHelmet");
#endif
#if defined USE_SPEED
	m_flLaggedMovementValue = UTIL_FindSendPropOffs("CCSPlayer", "m_flLaggedMovementValue");
#endif
#if defined USE_MONEY
	m_iAccount				= UTIL_FindSendPropOffs("CCSPlayer", "m_iAccount");
#endif
	
	g_bIsCSGO = (GetEngineVersion() == Engine_CSGO);

	g_hWaitRounds = CreateConVar("sm_vip_wait_rounds", "0", "Сколько первых раундов не выдавать привилегии (0 - отключено)", _, true, 0.0);
	g_hGiveHelmet = CreateConVar("sm_vip_give_helmet", "1", "Выдавать шлем при выдаче брони? (0 - нет)", _, true, 0.0);
	g_hWaitRounds.AddChangeHook(OnRoundsChange);
	g_hGiveHelmet.AddChangeHook(OnGiveHelmetChange);

	AutoExecConfig(true, "VIP_BaseFunctions", "vip");

	LoadTranslations("vip_modules.phrases");
	
	HookEventEx("round_start", Event_RoundStart, EventHookMode_PostNoCopy);
	HookEventEx("round_end", Event_RoundEnd, EventHookMode_PostNoCopy);
	
	if (g_bIsCSGO)
	{
		HookEvent("announce_phase_end", Event_Restart, EventHookMode_PostNoCopy);
		HookEventEx("cs_match_end_restart", Event_Restart, EventHookMode_PostNoCopy);
	}

	if (VIP_IsVIPLoaded())
	{
		VIP_OnVIPLoaded();
	}
}

public void OnPluginEnd()
{
	if (CanTestFeatures() && GetFeatureStatus(FeatureType_Native, "VIP_UnregisterMe") == FeatureStatus_Available)
	{
		VIP_UnregisterMe();
	}
}

int UTIL_FindSendPropOffs(const char[] sNetClass, const char[] sPropertyName)
{
	int iOffset;
	if ((iOffset = FindSendPropInfo(sNetClass, sPropertyName)) == -1)
	{
		SetFailState("Unable to find offset: \"%s::%s\"", sNetClass, sPropertyName);
	}

	return iOffset;
}

public void OnMapStart()
{
	g_iRounds = 0;
	if (g_bIsCSGO)
	{
		float fDuration = GetConVarFloat(FindConVar("mp_warmuptime"));
		if (fDuration > 0.0)
		{
			g_iRounds = -1;
			CreateTimer(fDuration, Timer_ResetRound, _, TIMER_FLAG_NO_MAPCHANGE);
		}
	}
}

public void OnConfigsExecuted()
{
	OnRoundsChange(g_hWaitRounds, NULL_STRING, NULL_STRING);
	OnGiveHelmetChange(g_hGiveHelmet, NULL_STRING, NULL_STRING);
}

public void OnRoundsChange(Handle hCvar, const char[] oldValue, const char[] newValue)
{
	g_iCvar_WaitRounds = GetConVarInt(hCvar);
	if (g_iCvar_WaitRounds < 0)
	{
		g_iCvar_WaitRounds = 0;
	}
}

public void OnGiveHelmetChange(Handle hCvar, const char[] oldValue, const char[] newValue)
{
	g_bGiveHelmet = GetConVarBool(hCvar);
}

public Action Timer_ResetRound(Handle hTimer)
{
	g_iRounds = 0;
}

public void Event_RoundEnd(Handle hEvent, const char[] sEvName, bool bDontBroadcast)
{
	if (g_iRounds == -1)
	{
		return;
	}
	
	char sReason[64];
	GetEventString(hEvent, "message", sReason, sizeof(sReason));
	if (strcmp(sReason, "#Game_Commencing") == 0)
	{
		g_iRounds = 0;
	}
}

public void Event_Restart(Handle hEvent, const char[] sEvName, bool bDontBroadcast) { g_iRounds = 0; }

public void Event_RoundStart(Handle hEvent, const char[] sEvName, bool bDontBroadcast)
{
	if (g_iRounds == -1)
	{
		return;
	}

	g_iRounds++;
}

public void VIP_OnVIPLoaded()
{
#if defined USE_HEALTH
	VIP_RegisterFeature(g_szFeature[VIP_HEALTH], STRING, _, _, OnItemDisplay);
#endif
#if defined USE_ARMOR
	VIP_RegisterFeature(g_szFeature[VIP_ARMOR],	STRING,	_, _, OnItemDisplay);
#endif
#if defined USE_SPEED
	VIP_RegisterFeature(g_szFeature[VIP_SPEED],	STRING,	_, OnItemToggle, OnItemDisplay);
#endif
#if defined USE_MONEY
	VIP_RegisterFeature(g_szFeature[VIP_MONEY],	STRING, TOGGLABLE, _, OnItemDisplay);
#endif
}

public void VIP_OnPlayerSpawn(int iClient, int iTeam, bool bIsVIP)
{
	if (!bIsVIP || g_iRounds == -1 || (g_iCvar_WaitRounds > 0 && g_iCvar_WaitRounds >= g_iRounds))
	{
		return;
	}

#if defined USE_HEALTH
	if (VIP_IsClientFeatureUse(iClient, g_szFeature[VIP_HEALTH]))
	{
		UTIL_GiveClientData(iClient, g_szFeature[VIP_HEALTH], m_iHealth);
	}
#endif

#if defined USE_ARMOR

	if (VIP_IsClientFeatureUse(iClient, g_szFeature[VIP_ARMOR]))
	{
		GivePlayerItem(iClient, "item_assaultsuit");
		if (g_bGiveHelmet)
		{
			SetEntData(iClient, m_bHasHelmet, 1);
		}
		UTIL_GiveClientData(iClient, g_szFeature[VIP_ARMOR], m_ArmorValue);
	}
#endif

#if defined USE_SPEED
	if (VIP_IsClientFeatureUse(iClient, g_szFeature[VIP_SPEED]))
	{
		char sValue[16];
		VIP_GetClientFeatureString(iClient, g_szFeature[VIP_SPEED], sValue, sizeof(sValue));

		float fSpeed;
		if (sValue[0] == '+')
		{
			fSpeed = StringToFloat(sValue[2]) + GetEntDataFloat(iClient, m_flLaggedMovementValue);
		}
		else
		{
			StringToFloatEx(sValue, fSpeed);
		}

		SetEntDataFloat(iClient, m_flLaggedMovementValue, fSpeed, true);
	}
#endif

#if defined USE_MONEY
	if(VIP_IsClientFeatureUse(iClient, g_szFeature[VIP_MONEY]))
	{
		UTIL_GiveClientData(iClient, g_szFeature[VIP_MONEY], m_iAccount);
	}
#endif
}

stock void UTIL_GiveClientData(int iClient, const char[] sFeatureName, int m_Offset)
{
	char sValue[16];
	VIP_GetClientFeatureString(iClient, sFeatureName, sValue, sizeof(sValue));

	int iValue;
	if (sValue[0] == '+')
	{
		iValue = StringToInt(sValue[2]) + GetEntData(iClient, m_Offset);
	}
	else
	{
		StringToIntEx(sValue, iValue);
	}

	SetEntData(iClient, m_Offset, iValue);
}

public Action OnItemToggle(int iClient, const char[] szFeature, VIP_ToggleState eOldStatus, VIP_ToggleState &eNewStatus)
{
	if (eNewStatus == DISABLED)
	{
		if (strcmp(szFeature, g_szFeature[VIP_SPEED]) == 0)
		{
			SetEntDataFloat(iClient, m_flLaggedMovementValue, 1.0, true);
		}
	}

	return Plugin_Continue;
}

public bool OnItemDisplay(int iClient, const char[] sFeatureName, char[] sDisplay, int iMaxLength)
{
	if (!VIP_IsClientFeatureUse(iClient, sFeatureName))
	{
		return false;
	}

	char sValue[16];
	VIP_GetClientFeatureString(iClient, sFeatureName, sValue, sizeof(sValue));
	int iStartIndex = (sValue[0] == '+') ? 1 : 0;

	if(strcmp(sFeatureName, g_szFeature[VIP_SPEED]) == 0)
	{
		FormatEx(sDisplay, iMaxLength, "%T [x%.2f]", sFeatureName, iClient, StringToFloat(sValue[iStartIndex]));
	}
	else
	{
		FormatEx(sDisplay, iMaxLength, "%T [%s]", sFeatureName, iClient, sValue[iStartIndex]);
	}
	return true;
}
